# Modern Applied Biostatistical Methods Using S-PLUS
# S. Selvin, Oxford, 1998
#
# Chapter 2: Descriptive Techniques
#
# Time-stamp: <2008-03-10 22:49:06 chl>

describe <- function(x) {
  n <- length(x)
  xbar <- mean(x)
  md <- median(x)
  s2 <- var(x)
  s <- sqrt(s2)
  se <- s/sqrt(n)
  xmin <- min(x)
  xmax <- max(x)
  rng <- xmax-xmin
  cbind(n,xbar,md,s2,s,se,xmin,xmax,rng)
}

###
### not in the book
bwt <- scan("bwt.txt")
###

describe(bwt)

summary(bwt)

ints <- seq(0,6000,500)
table(cut(bwt,ints))

                                        # Fig. 2.1
###
### not in the book
hist(bwt,breaks=seq(0,6000,by=500),main="n = 750 birth weight (parity = 1)")
###

                                        # Fig. 2.2
hist(bwt,nclass=50,main="n = 750 birth weight (parity = 1)")

                                        # Fig. 2.3
plot(density(bwt),type="l",main="n = 750 birth weight (parity = 1)")

count <- c(815,319,123,171)
                                        # Fig. 2.4 (left)
### - R --
### set density and col parameters
barplot(count,names=c("white","hispanic","black","asian"),
        angle=c(45,30,15,0),density=3,col=1,xlab="race/ethnic group",
        main="Barplot")
###
barplot(count,names=c("white","hispanic","black","asian"),
        angle=c(45,30,15,0),xlab="race/ethnic group",
        main="Barplot")

percent <- 100*count/sum(count)
round(percent,1)
                                        # Fig. 2.4 (right)
### - R --
### replace names= with labels=
pie(percent,names=c("white","hispanic","black","asian"),
    density=0,xlab="race/ethnic group",main="Pie chart")

x <- c(2.3,0.8,2.0,1.2,1.3,0.9,3.8,2.2,5.5,1.0,3.4,5.3,1.1,0.7,2.7)
n <- length(x)
dx <- 2
z <- seq(min(x)-dx,max(x)+dx,0.1)
y <- matrix(0,n,length(z))
h <- 1.0
for (i in 1:n) { y[i,] <- dnorm(z,x[i],h) }
zfreq <- apply(y,2,sum)
plot(z,zfreq,xlab="x -- observed values",xlim=c(0,6),ylim=c(0,4),type="l")

                                        # Fig. 2.5
###
### not in the book
plot(z,zfreq+2,xlab="x -- observed values",ylab="",xlim=c(0,6),ylim=c(0,8),type="l")
abline(h=c(0,2.2))
for (i in 1:n) { lines(z,y[i,]*2/0.4,lty=3) }
legend(2.2,7,c("density estimate","kernel functions"),lty=c(1,3))
title("Illustration of a density plot")
###

plot(density(x,width=3.5),xlab="x -- observed values",xlim=c(0,6),
     ylim=c(0,0.3),ylab="frequency",type="l")

                                        # Fig. 2.6
###
### not in the book
opar <- par(mfrow=c(2,4))
z <- rnorm(100)
h <- c(0.01,0.03,0.05,0.1,0.2,0.3,0.4,0.5)
for (i in 1:length(h))
  plot(density(z,h[i]),type="l",xlab="z",ylab="y",main=paste("h =",h[i]))
par(opar)
###

###
### not in the book
bwt <- read.table("bwt2.txt",header=TRUE)
bwtw <- subset(bwt,race==1)$bwt
bwtb <- subset(bwt,race==2)$bwt
###

stem(bwtw)
stem(bwtb)

round(describe(bwt),4)
round(describe(bwtw),4)
round(describe(bwtb),4)

t.test(bwtw,bwtb)

var.test(bwtw,bwtb)

five <- function(data) {
  x <- sort(data)
  n <- length(x)
  m <- (n+1)/2
  fm <- 0.5*(x[ceiling(m)]+x[trunc(m)])
  m <- (trunc(m)+1)/2
  fl <- 0.5*(x[ceiling(m)]+x[trunc(m)])
  m <- n+1-m
  fu <- 0.5*(x[ceiling(m)]+x[trunc(m)])
  upper <- fu+1.5*(fu-fl)
  lower <- fl-1.5*(fu-fl)
  cbind(lower,fl,fm,fu,upper)
}

data <- c(0.51,0.01,0.08,0.09,0.14,0.20,0.01,0.09,0.06,0.05,0.01,
          0.03,0.05,0.22,0.68)
### - R --
### fivenum(data)
five(data)
quantile(data)

                                        # Fig. 2.7
boxplot(data,main="Benzene exposure data")

                                        # Fig. 2.8
boxplot(bwtw,bwtb,names=c("whites","blacks"),ylab="birth weight",
        main="Birth weight distributions -- whites versus blacks")


###
### not in the book
rdata.mat <- matrix(c(-2.293,-1.349,-1.005,-0.683,-0.434,-0.235,-0.204,-0.193,
                      -0.117,-0.102,-0.010,0.013,0.128,0.235,0.250,0.254,
                      0.771,0.832,1.932,2.209,
                      0.05,0.10,0.15,0.20,0.25,0.30,0.35,0.40,
                      0.45,0.50,0.55,0.60,0.65,0.70,0.75,0.80,
                      0.85,0.90,0.95,1.00,
                      0.011,0.089,0.158,0.247,0.332,0.407,0.419,0.423,
                      0.453,0.460,0.496,0.505,0.551,0.593,0.599,0.600,
                      0.780,0.797,0.973,0.986),nc=3)
rdata <- rdata.mat[,1]
###

                                        # Fig. 2.9
###
### not in the book
plot(rdata.mat[,1],rdata.mat[,2],type="s",xlab="x",ylab="",ylim=c(0,1),
     main="Cumulative distributions")
lines(rdata.mat[,1],rdata.mat[,3],lty=3)
legend(1,0.4,c("empirical","theoretical"),lty=c(1,3))
###


                                        # Fig. 2.10
###
### not in the book
qqnorm(sort(rdata),type="l")
qqline(sort(rdata))
###

n <- length(rdata)
y0 <- sort(rdata)
x0 <- qnorm(((1:n)-1/3)/(n+1/3))
plot(x0,y0,type="l")

y1 <- pnorm(sort(rdata))
x1 <- ((1:n)-1/3)/(n+1/3)
plot(x1,y1,type="l")

###
### not in the book
bwt <- scan("bwt.txt")
###

                                        # Fig. 2.11
### - R --
### z <- scale(bwt)
z <- sort((bwt-mean(bwt))/sqrt(var(bwt)))
qqnorm(z,type="l",main="n = 750 birthweight (parity = 1)")
qqline(z)

nbwt <- bwt[bwt>2500]
describe(nbwt)

                                        # Fig. 2.12
###
### not in the book
nz <- sort((nbwt-mean(nbwt))/sqrt(var(nbwt)))
qqnorm(nz,type="l",main="n = 692 birthweight (bwt > 2500g and parity = 1)")
qqline(nz)
###

                                        # Fig. 2.13
###
### not in the book
opar <- par(mfrow=c(2,3))
hist(bwt,breaks=50,main="n = 750 birthweight (parity = 1)")
plot(density(bwt,width=280),xlab="bwt",ylab="",main="n = 750 birthweight (parity = 1)")
qqnorm(z,type="l",main="n = 750 birthweight (parity = 1)")
qqline(z)
hist(nbwt,breaks=50,main="n = 692 birthweight (bwt > 2500g and parity = 1)")
plot(density(nbwt,width=150),xlab="bwt",ylab="",main="n = 692 birthweight (bwt > 2500g and parity = 1)")
qqnorm(nz,type="l",main="n = 692 birthweight (bwt > 2500g and parity = 1)")
qqline(nz)
par(opar)
###

                                        # Fig. 2.14
###
### not in the book
z1 <- rdata.mat[,1:2]
z2 <- matrix(c(-1.958,-1.073,-0.949,-0.760,-0.709,-0.676,-0.546,
               -0.508,-0.192,-0.104,-0.061,0.005,0.046,0.077,0.138,
               1.114,1.125,1.635,1.669,1.725,seq(0.05,1,by=.05)),nc=2)
plot(z1[,1],z1[,2],type="s",xlab="z-value",ylab="probability",ylim=c(0,1))
lines(z2[,1],z2[,2],type="s",lty=3)
abline(h=c(0,1))
legend(-2,0.8,c("sample z1","sample z2"),lty=c(1,3))
title(main="Cumulative distributions")
###

                                        # Fig. 2.15
###
### not in the book
qqplot(z1[,1],z2[,1],xlab="z1",ylab="z2",type="l")
abline(0,1)
title(main="S-qqplot -- z1 versus z2")
###
qqplot(z1,z2,type="l",main="S-qqplot -- z1 versus z2")

xx <- sort(x)
yy <- sort(y)
plot(xx,yy,type="l")
abline(0,1)

cdf <- ((1:n)-1/3)/(n+1/3)
qqplot(qnorm(cdf),z,type="l")

### Not run
###
qqplot(qnorm(cdf,m,s),x,type="l")
###

### Not run
###
qqplot(qexp(cdf,lam),sdata,type="l")
###

                                        # Fig. 2.16
plot(density(bwtw),type="l",ylab="density - bwt",xlab="birth weight",
     main="Birth weigth distributions -- whites versus blacks")
### - R --
### lines(density(bwtb))
par(new=T)
plot(density(bwtb),xlab="",ylab="",axes=F,type="l",lty=2)

                                        # Fig. 2.17
plot(density(bwtw,width=1000),type="l",ylab="density - bwt",
     xlab="birth weight",
     main="Birth weigth distributions -- whites versus blacks")
### - R --
### lines(density(bwtb))
par(new=T)
plot(density(bwtb,width=1000),xlab="",ylab="",axes=F,type="l",lty=2)

                                        # Fig. 2.18
qqplot(bwtw,bwtb,type="l",main="qqplot -- white versus black")
abline(0,1)

###
### not in the book
exw <- read.table("exlife-urban.txt",header=TRUE)
attach(exw)
###


                                        # Fig. 2.19
plot(exwf[urban<75],exwm[urban<75],pch="+",ylim=range(exwm),xlim=range(exwf),
     ylab="expectation of life (wm)",xlab="expectation of life (wf)",
     main="Expectation of life -- males verss females")
points(exwf[urban>=75],exwm[urban>=75],pch="o")
abline(lsfit(exwf,exwm)$coef)
points(mean(exwf[urban<75]),mean(exwm[urban<75]),pch="x",cex=1.5)
points(mean(exwf[urban>=75]),mean(exwm[urban>=75]),pch="*",cex=1.5)
legend(74.7,66,
       c("%urban<75","%urban>=75","non-urban centroid","urban centroid"),
       pch="+ox*")

###
### not in the book
detach(exw)
###

                                        # Fig. 2.20
ht <- seq(130,200,1)
wt <- seq(40,110,1)
### - R --
### added theta=-45 which otherwise defaults to 0
bwt <- outer(ht,wt,function(ht,wt) {7627-72.0*ht+25.0*wt+0.25*ht^2-0.13*wt^2})
persp(ht,wt,bwt/1000,theta=-45,xlab="height",ylab="weight",zlab="birth weight")
title("Birth weight -- maternal height by weight")


                                        # Fig. 2.21
par(mfrow=c(2,2))
par(cex=0.4)
### - R --
### change eye=c(x,y,z) to theta and phi parameters
persp(ht,wt,bwt/1000,theta=-10,phi=5,ticktype="detailed",
      xlab="ht",ylab="wt",zlab="bwt",expand=.75)
persp(ht,wt,bwt/1000,theta=-135,phi=15,ticktype="detailed",
      xlab="ht",ylab="wt",zlab="bwt")
persp(ht,wt,bwt/1000,theta=-45,phi=15,ticktype="detailed",
      xlab="ht",ylab="wt",zlab="bwt")
persp(ht,wt,bwt/1000,theta=-70,phi=0,ticktype="detailed",
      xlab="ht",ylab="wt",zlab="bwt")

                                        # Fig. 2.22
par(pty="s")
contour(x,y,z,xlim=c(1700,3100),ylim=c(1400,2700))
title("Contours of emf exposure from Sutro tower -- milligauss")

zht <- function(x,y) {exp(-0.5*(((x-mx)^2/vx-2*r*(x-mx)*(y-my)/sqrt(vx*vy)+(y-my)^2/vy)/(1-r^2)))/sqrt(2*pi*vx*vy*(1-r^2))}

                                        # Fig. 2.23
mx <- 0
vx <- 1
dx <- 0.1
x <- seq(mx-3*sqrt(vx),mx+3*sqrt(vx),dx)
my <- 0
vy <- 1
dy <- 0.1
y <- seq(my-3*sqrt(vy),my+3*sqrt(vy),dy)
r <- 0.5
z <- outer(x,y,zht)
contour(x,y,z,nint=10,pty="s",xlab="x",ylab="y")
title("Contours for a bivariate normal distribution
m={0,0} and v={1,1} with r = 0.5")

###
### not in the book
gst <- read.table("gestation.txt",header=TRUE)
x <- gst$gest
y <- gst$pattern
z <- gst$gain
###

                                        # Fig. 2.24
contour(ht,wt,bwt/1000,nint=16,labex=T,
        xlab="height (centimeters)",ylab="weight (kilograms)")
title("bwt -- maternal height by weight")

                                        # Fig. 2.25
x0 <- (x-mean(x))/sqrt(var(x))
y0 <- (y-mean(y))/sqrt(var(y))
z0 <- (z-mean(z))/sqrt(var(z))
pairs(cbind(x0,y0,z0),pch="*",cex=0.75)

                                        # Fig. 2.26
tx <- seq(0,360,45)
par(pty="s")
par(mfrow=c(3,3))
for (i in 1:length(tx)) {
  t0 <- pi*tx[i]/180
  znew <- z0
  ynew <- x0*cos(t0)+y0*sin(t0)
  xnew <- -x0*sin(t0)+y0*cos(t0)
  temp <- paste("rotation in degrees =",tx[i])
  plot(ynew,znew,pch=".",xlab="",ylab="",xlim=c(-4,4),ylim=c(-4,4),main=temp)
  lines(c(0,0),c(0,2))
  lines(c(0,2*cos(t0)),c(0,0))
  lines(c(0,2*cos(t0+pi/2)),c(0,0))
  text(0,2,"Z")
  text(2*cos(t0),-0.3,"Y")
  text(2*cos(t0+pi/2),0.3,"X")
}

###
### not in the book
leuk <- read.table("leukemia.txt",header=TRUE)
attach(leuk)
age <- seq(0,85,length=18)
###

                                        # Fig. 2.27
plot(age,rate,type="b",ylim=range(rate),ylab="female leukemia rates",
     main="Female leukemia incidence rates per 100,000, California, 1987")
lines(age,smooth(rate),type="l",lty=2)

###
### not in the book
detach(leuk)
###

###
### not in the book
cc <- read.table("colon-cancer.txt",header=TRUE)
###

                                        # Fig. 2.28
###
### not in the book
ff <- matrix(c(cc$n[1:12]/sum(cc$n[1:12]),cc$n[13:24]/sum(cc$n[13:24])),nc=2)
opar <- par(mfrow=c(2,1),cex.axis=.7)
plot(cc$year[1:12],ff[,1],type="l",xlab="year",ylab="proportion of cases",ylim=c(0,0.2))
lines(cc$year[1:12],ff[,2],lty=3)
title(main="Colon cancer frequency\nblack females, Oakland, CA")
plot(cc$year[1:12],smooth(ff[,1]),type="l",xlab="year",ylab="proportion of cases",ylim=c(0,0.2))
lines(cc$year[1:12],smooth(ff[,2]),lty=3)
par(opar)
###

                                        # Fig. 2.29
n <- 100
x0 <- runif(n,0,10)
y0 <- x0+runif(n,0,10)
xtemp <- order(x0)
y <- y0[xtemp]
x <- x0[xtemp]
plot(x,y,pch=".",main="Four S-smoothing options")
lines(x,smooth(y),lty=1)
lines(supsmu(x,y),lty=2)
lines(loess.smooth(x,y),lty=3)
lines(smooth.spline(x,y),lty=4)
legend(1,15,c("smooth","supsmu","loess.smooth","smooth.spline"),lty=1:4)


                                        # Fig. 2.30
k <- length(x0)
x <- seq(1,10,0.4)
y <- seq(1,10,0.4)
n <- length(x)
z <- array(0,c(n,n,k))
h <- 1.0
zfcn <- function(x,y) {(exp(-0.5*(((x-m1)/h)^2+((y-m2)/h)^2)))/(2*pi)}
for (i in 1:k) {
  m1 <- x0[i]
  m2 <- y0[i]
  z[,,i] <- outer(x,y,zfcn)
}
zsum <- apply(z,c(1,2),sum)
par(pty="s")
par(mfrow=c(2,2))
plot(x0,y0,pch="*",xlim=c(1,10),ylim=c(1,10))
title("Hypothetical data")
persp(x,y,zsum/max(zsum),cex=0.5)
title("Perspective plot")
contour(x,y,zsum,cex=0.7,nint=6)
title("Contour plot")
contour(x,y,zsum,cex=0.7,levels=0.8)
title("Areas of high incidence")

###
### not in the book
expos <- read.table("exposure.txt",header=FALSE)
x <- expos
dimnames(expos) <- list(c("attendant","transportation 1","transportation 2",
                          "outside","inside","maintenance"),
                        c("n-butene","isopentane","n-pentane","isobutane",
                          "n-hexane","methylbutene"))
###

                                        # Fig. 2.32
### - R --
### dist(x,method="euclidean")
dd <- dist(x,metric="euclidean")
### - R --
### hclust(dd,method="complete")
htemp <- hclust(dd,method="connected")
ltemp <- c("attendants","transport 1","transport 2","outside","inside",
           "maintenance")
plclust(htemp,labels=ltemp,main="Exposure to gasoline components")

###
### not in the book
rm(x)
###

###
### not in the book
cereal <- read.csv("cereal.csv",header=TRUE,sep=",")
cdata <- cereal[,3:13]
cnames <- cereal$name
###

                                        # Fig. 2.33
dd <- dist(cdata,metric="euclidean")
htemp <- hclust(dd,method="connected")
par(mfrow=c(1,2))
plclust(htemp,cex=0.5,labels=cnames,main="cereal classification")
plclust(htemp,cex=0.5,labels=grains,main="cereal classification")

                                        # Fig. 2.34
faces(cdata,labels=cnames)

                                        # Fig. 2.35
starts(cdata,labels=cnames)

a <- matrix(1:35,5,7)
a
sweep(a,2,10,"/")
sweep(a,2,c(1,2,3,4,5,6,7),"/")
mtemp <- apply(a,1,mean)
mtemp
sweep(a,1,mtemp,"-")

### not in the book
x <- matrix(c(6,13,11,3,16,14,12,19,11,9,22,22),nr=3)
###

xx <- x
xx <- sweep(xx,1,apply(xx,1,mean))
xx <- sweep(xx,2,apply(xx,2,mean))
round(xx,4)

round(x-xx,4)

xx <- x
for (i in 1:5) {
  xx <- sweep(xx,1,apply(xx,1,median))
  xx <- sweep(xx,2,apply(xx,2,median))
}
round(xx,4)

round(x-xx,4)

### - R --
### medpolish(x,iter=5)$residuals
twoway(x,iter=5)$resid

### not in the book
x <- read.table("chd.txt",header=TRUE)
x$CHD <- as.factor(x$CHD)
###

x
xx <- sweep(x,1,apply(x,1,mean))
xx <- sweep(xx,2,apply(xx,2,mean))
round(xx,3)
round(x-xx,3)

xx <- log(x)
xx <- sweep(xx,1,apply(xx,1,mean))
xx <- sweep(xx,2,apply(xx,2,mean))
round(exp(xx),3)
round(exp(log(x)-xx),3)

                                        # Fig. 2.36

                                        # Fig. 2.37
x <- c(3,3,1,6,11,13,23,23,8)
y <- c(8,13,18,18,10,8,8,1,1)
par(pty="s")
plot(x,y,type="n",main="A California county")
polygon(x,y,density=10)
sx <- (c(x,x[1])+c(1,x))[-1]
sy <- (c(x,y[1])+c(1,y))[-1]
dx <- (c(x,x[1])-c(1,x))[-1]
dy <- (c(x,y[1])-c(1,y))[-1]
w <- sy*dx-sx*dy
area <- sum(w)/4
p <- sum(sqrt(dx^2+dy^2))
text(3.5,2,paste("area =",round(area,3)))
text(3.5,3,paste("perimeter =",round(p,3)))
ex <- sum(w*sx)/(12*area)
ey <- sum(w*sy)/(12*area)
points(ex,ey,pch="*",cex=1.5)
text(ex+2,ey+2,"centroid")
lines(c(ex,ex+1.7),c(ey,ey+1.7))

                                        # Fig. 2.38
### not in the book
par(mfrow=c(2,2))
###
n <- 100
x <- rnorm(n)
y <- rnorm(n)
plot(x,y,pch="*")
hpoints <- chull(x,y)
polygon(x[hpoints],y[hpoints],density=0)

for (i in 1:100) {
  x0 <- x
  y0 <- y
  hpoints <- chull(x,y)
  x <- x[-hpoints]
  y <- y[-hpoints]
  if (length(x)<=3) break
}
polygon(x0[hpoints],y0[hpoints],density=0)

                                        # Fig. 2.39
par(mfrow=c(3,2))
x <- seq(-2*pi,2*pi,0.25)
y <- x
z <- outer(x,y,function(x,y) {sin(x)*cos(y)})
persp(x,y,z,box=F,axes=F)
title("sin*cos")

x1 <- seq(-0.1,0.001,0.001)
x2 <- seq(0.001,0.1,0.001)
x <- c(x1,x2)
y <- x*sin(1/x)
plot(x,y,type="l",main="x*sin(1/x)")
abline(h=0)
abline(v=0)

x <- seq(-2*pi,2*pi,0.01)
y <- 2*(1-4*sin(x))
plot(y*cos(x),y*sin(x),type="l",main="limacon")

x <- seq(-1,1,0.1)
y <- x
z <- outer(x,y,function(x,y) {x^2/4-y^2/4})
persp(x,y,z)
title("hyperbolic paraboloid")


tt <- seq(0,24*pi,len=2000)
rr <- exp(cos(tt))-2*cos(4*tt)+sin(tt/12)^5
plot(rr*sin(tt),-rr*cos(tt),type="l",axes=F,xlab="butterfly",ylab="")

x <- seq(-2*pi,2*pi,0.01)
y <- 4*sin(8*x)
par(pty="s")
plot(y*cos(x),y*sin(x),type="l",main="leaved flower")

### end of file 2.R
