# Modern Applied Biostatistical Methods Using S-PLUS
# S. Selvin, Oxford, 1998
#
# Chapter 6: Analysis of Tabular Data
#
# Time-stamp: <2008-08-02 22:14:49 chl>

x1 <- c(26,21)
n1 <- c(64,65)
d1 <- rbind(x1,n1-x1)
d1
chisq.test(d1)

prop.test(d1)

fisher.test(d1)

N <- 20
m <- 10
n <- 6
x0 <- 2
x <- max(0,n+m-N):min(m,n)
p <- dhyper(x,m,N-m,n)
round(p,4)
p0 <- dhyper(x0,m,N-m,n)
cat("x0 =",x0," p0 =",round(p0,4),"\n")
pvalue <- sum(p[p<=p0])
cat("p-value =",round(pvalue,4),"\n")

a <- cbind(c(x0,m-x0),c(n-x0,(N-m-n+x0)))
a
fisher.test(a)

x0 <- c(18,13)
n0 <- c(50,50)
dtest <- array(c(x1,n1-x1,x0,n0-x0),c(2,2,2))
dtest
mantelhaen.test(dtest)

or.mh <- function(datatab) {
  a0 <- datatab[1,1,]
  b0 <- datatab[1,2,]
  c0 <- datatab[2,1,]
  d0 <- datatab[2,2,]
  n0 <- a0+b0+c0+d0
  sum(a0*d0/n0)/sum(b0*c0/n0)
}
or.mh(dtest)

m <- matrix(c(15,22,40,90),2,2)
m
mcnemar.test(m)

binom.test(m[1,2],(m[1,2]+m[2,1]),p=0.5)

r1 <- c(7332,287,199,96,59,65)
r2 <- c(7673,239,154,65,28,29)
d <- rbind(r1,r2)
d
chisq.test(d)

# convert table into n paired observations
x <- c(rep(0:(length(r1)-1),r1),rep(0:(length(r2)-1),r2))
y <- c(rep(1,sum(r1)),rep(0,sum(r2)))
table(y,x)
sxx <- sum((x-mean(x))^2)
syy <- sum((y-mean(y))^2)
sxy <- sum((y-mean(y))*(x-mean(x)))
# chi-square test for trend
b <- sxy/sxx
b
# or b <- lsfit(x,y)$coef[2]
n <- sum(d)
# linear chi-square
v <- syy/((n-1)*sxx)
linear <- b^2/v
p.linear <- 1-pchisq(linear,1)
round(cbind(linear,p.linear),3)

# total chi-square
total <- chisq.test(d)$statistic
p.total <- chisq.test(d)$p.value
# nonlinearity chi-square
nonlinear <- total-linear
p.nl <- 1-pchisq(nonlinear,length(r1)-2)
round(cbind(linear,nonlinear,total),3)
round(cbind(p.nl,p.linear,p.total),3)

# t-like test
dd <- mean(x[y==1])-mean(x[y==0])
vdd <- sxx*(1/sum(r1)+1/sum(r2))/(n-1)
z <- dd/sqrt(vdd)
pvalue <- 2*(1-pnorm(abs(z)))
round(cbind(z,pvalue),3)

x <- rbind(c(42,22),c(32,81))
x
or <- x[1,1]*x[2,2]/(x[1,2]*x[2,1])
round(or,3)
Y <- (x[1,1]*x[2,2]-x[1,2]*x[2,1])/(x[1,1]*x[2,2]+x[1,2]*x[2,1])
round(Y,3)
chi2 <- chisq.test(x,correct=F)$statistic
round(chi2,3)
n <- sum(x)
PHI <- sqrt(chi2/n)
round(PHI,3)
C <- sqrt(chi2/(chi2+n))
round(C,3)

xtemp <- c(42,18,4,0,12,22,8,2,18,6,10,6,2,8,12,7)
x <- matrix(xtemp,4,byrow=T)
x
n <- sum(x)
chisq.test(x,correct=F)
chi2 <- chisq.test(x,correct=F)$statistic
V0 <- sqrt(chi2/(n*min((nrow(x)-1),(ncol(x)-1))))
T0 <- sqrt(chi2/(n*sqrt((nrow(x)-1)*(ncol(x)-1))))
PHI <- sqrt(chi2/n)
C <- sqrt(chi2/(chi2+n))
round(cbind(V0,T0,C,PHI),3)

lam.cor <- function(x) {
  n <- sum(x)
  xmax <- apply(x,2,max)
  cmax <- max(apply(x,1,sum))
  rcol <- (sum(xmax)-cmax)/(n-cmax)
  xmax <- apply(x,1,max)
  rmax <- max(apply(x,2,sum))
  rrow <- (sum(xmax)-rmax)/(n-rmax)
  r <- ((n-rmax)*rrow+(n-cmax)*rcol)/(2*n-rmax-cmax)
  cbind(rcol,rrow,r)
}
round(lam.cor(x),3)

# convert x (4 by 4) into n = 177 paired observations
a0 <- rep(1:nrow(x),apply(x,1,sum))
b0 <- rep(rep(1:ncol(x),nrow(x)),array(t(x)))
table(a0,b0)
# count the pair types
a <- outer(a0,b0,"-")
b <- outer(b0,b0,"-")
P <- sum(ifelse(a*b>0,1,0))/2
Q <- sum(ifelse(a*b<0,1,0))/2
Ta <- (sum(ifelse(a==0,1,0))-sum(x^2))/2
Tb <- (sum(ifelse(b==0,1,0))-sum(x^2))/2
cbind(P,Q,Ta,Tb)
# Goodman and Kruskal (gamma)
(P-Q)/(P+Q)
# Kendal's tau
2*(P-Q)/sqrt((P+Q+Ta)*(P+Q+Tb))
# Sommer's r=b|a
(P-Q)/(P+Q+Tb)
# Sommer's r=a|b
(P-Q)/(P+Q+Ta)

# Pearson correlation
cor(a0,b0)

tau <- function(x,y) {
  x0 <- outer(x,x,"-")
  y0 <- outer(y,y,"-")
  p <- sum(ifelse(x0*y0>0,1,0))
  q <- sum(ifelse(x0*y0<0,1,0))
  (p-q)/(p+q)
}
# the vector m contains the 12 male twin birth weights
m <- c(2850,2530,2590,2130,3180,2730,3040,2270,2360,2440,2820,3130)
# the vector f contains the 12 female twin birth weights
f <- c(2140,2531,2580,2370,3220,2970,3080,2820,2690,2470,2280,3000)
round(tau(m,f),3)

# f = vector containing the cell frequencies
f <- c(191,1120,422,4443)
x1 <- c(1,1,0,0)
x2 <- c(1,0,1,0)
otemp1 <- glm(f~x1+x2+x1*x2,family=poisson)
summary(otemp1,correlation=F)

exp(predict(otemp1))

otemp0 <- glm(f~x1+x2,family=poisson)
summary(otemp0,correlation=F)

exp(predict(otemp0))

sum(residuals(otemp0,type="deviance")^2)

sum(residuals(otemp0,type="deviance")^2)

chisq.test(matrix(c(191,422,1120,4443),2,2))

# f = vector containing the cell frequencies
f <- c(118,866,288,3672,73,254,134,771)
x1 <- c(1,1,0,0,1,1,0,0)
x2 <- c(1,0,1,0,1,0,1,0)
x3 <- c(0,0,0,0,1,1,1,1)
otemp1 <- glm(f~x1+x2+x3+x1*x2+x1*x3+x2*x3+x1*x2*x3,family=poisson)
summary(otemp1,correlation=F)

otemp0 <- glm(f~x1+x2+x3+x1*x2+x1*x3+x2*x3,family=poisson)
summary(temp0,correlation=F)

sum(residuals(otemp0,type="pearson")^2)

ex <- exp(predict(otemp0))
# f = observed and ex = expected frequencies
round(cbind(x1,x2,x3,f,ex),3)

c1 <- c(1,2,3,4,1,2,3,4,1,2,3,4,1,2,3,4)
c1
c1+3
f1 <- factor(c1)
f1
f1+3

###
### not in the book
blood <- read.table("blood.txt",header=T)
names(blood)[1:2] <- c("mvalues","fvalues")
###

# count = the 32 cell frequencies (vector)
mother <- factor(mvalues)
father <- factor(fvalues)
otemp4 <- glm(count~moter+father+sex+mother*father+mother*sex+father*sex,family=poisson)
summary(otemp4,correlation=F)

1-pchisq(otemp4$deviance,otemp4$df.residual)

excount <- exp(predict(otemp4))
x2 <- sum((count-excount)^2/excount)
df <- otemp4$df.residual
pvalue <- 1-pchisq(x2,df)
round(cbind(x2,df,pvalue),3)

otemp1 <- glm(count~mother+father+sex,family=poisson)
otemp2 <- glm(count~mother+father+sex+mother*father,family=poisson)
otemp3 <- glm(count~mother+father+sex+mother*father+mother*sex,family=poisson)
otemp4 <- glm(count~mother+father+sex+mother*sex+father*sex,family=poisson)
anova(otemp1,otemp2,otemp3,otemp4)

ex <- exp(predict(otemp1))
rvalues <- residuals(otemp1,type="pearson")
round(cbind(mother,father,sex,count,ex,rvalues),3)

###
### not in the book
counts <- as.numeric(scan("births.dat",what="integer",sep=","))
race <- factor(rep(rep(1:4,each=4),4),labels=c("white","black","hispanic","asian"))
sex <- factor(rep(c(1,1,2,2),16),labels=c("male","female"))
wht <- factor(rep(1:2,32),labels=c(">=2500","<2500"))
gest <- factor(rep(1:2,each=32),labels=c(">=38","<38"))
parity <- factor(rep(rep(1:2,each=16),2),labels=c("0",">=1"))
births <- data.frame(counts=counts,race=race,sex=sex,wht=wht,gest=gest,parity=parity)
#otemp <- glm(counts~race*sex*wht*gest*parity,data=births,family=poisson)
###

b <- summary(otemp)$coefficients[,1]
z <- summary(otemp)$coefficients[,3]
zrank <- rank(-abs(z))
