# Modern Applied Biostatistical Methods Using S-PLUS
# S. Selvin, Oxford, 1998
#
# Chapter 8: Rates, Life Tables, and Survival
#
# Time-stamp: <2008-03-22 09:40:45 chl>

###
### - R --
### data are also available in the data.frame "adjust.txt"
temp <- scan("adjust.data")

###
### - R --
### add option `byrow=T` and transpose the matrix
d <- t(matrix(temp,8,byrow=T))
base <- 100000
d60 <- d[1,]
pop60 <- d[2,]
d40 <- d[3,]
pop40 <- d[4,]
c60 <- base*sum(d60)/sum(pop60)
c40 <- base*sum(d40)/sum(pop40)
cat("\n","1960 crude rate/100,000 =",round(c60,1),
    "\n","1940 crude rate/100,000 =",round(c40,1),"\n")
a40.direct <- base*sum((d40/pop40)*pop60)/sum(pop60)
cat("\n","direct adjusted 1940 rate/100,000 =",round(a40.direct,1),"\n")

smr <- sum(d40)/sum((d60/pop60)*pop40)
smr
a40.indirect <- c60*smr
cat("\n","indirect adjusted 1940 rate/100,000 =",round(a40.indirect,1),"\n")

g0 <- c(rep(0,length(d60)),rep(1,length(d40)))
d0 <- c(d60,d40)
pop0 <- c(pop60,pop40)
age <- seq(20,90,10)
a0 <- factor(c(age,age))
otemp <- glm(d0~g0+a0+offset(log(pop0)),family=poisson)
b <- otemp$coefficient[2]
smr0 <- exp(b)
cat("\n","model adjusted rate =",round(c60*smr0,2),"\n")

sd <- summary(otemp)$coefficients[2,2]
upper <- exp(b+1.96*sd)
lower <- exp(b-1.96*sd)
smr0 <- exp(b)
cbind(lower,smr0,upper)


                                        # Fig. 8.1

###
### not in the book

###

par(pty="s")
par(cex=0.75)
l40 <- log(d40/pop40)
l60 <- log(d60/pop60)
plot(age,l60,ylim=c(-11,-4),type="n",xlab="age",ylab="log-rate")
points(age,l40,lty=2,type="b",pch="*")
points(age,l60,lty=2,type="b",pch="o")
lines(age,predict(otemp)[g0==0])
lines(age,predict(otemp)[g0==1])
title("Log-rates by age for 1940 and 1960")
legend(40,-9,c("estimated","observed"),lty=1:2)

delta <- c(1,4,rep(5,17))
temp <- scan("life.data")
pop <- temp[seq(1,length(temp),2)]
deaths <- temp[seq(2,length(temp),2)]
n <- length(pop)
R <- deaths/pop
qx <- delta*R/(1+0.5*delta*R)
qx[n] <- 1
lx <- 100000*c(1,cumprod(1-qx))
d <- -diff(lx)
lx <- lx[-(n+1)]
L <- delta*(lx-0.5*d)
L[n] <- lx[n]/R[n]
Tx <- rev(cumsum(rev(L)))
e <- Tx/lx
out <- cbind(round(qx,4),round(lx,0),round(L,0),round(Tx,0),round(e,1))
ages <- c("0-1","1-4","5-9","10-14","15-19","20-24","25-29","30-34","35-39",
          "40-44","45-49","50-54","55-59","60-64","65-69","70-74","75-79",
          "80-84","85+")
matrix(out,n,5,dimnames=list(ages,c("qx","lx","Lx","Tx","ex")))

                                        # Fig. 8.2
###
### not in the book
xx <- seq(0,85,length=19)
plot(c(xx,90),c(lx/lx[1],0),type="b",pch=19,ylim=c(0,1),
     xlab="age in years",ylab="probabilities")
points(xx,qx,type="b",pch="+",lty=3)
title("Life table -- Mortality pattern California males, 1980")
###

                                        # Fig. 8.3
###
### not in the book
us.mortality <- read.table("USmortality.txt",header=T)

life.table <- function(p,d,delta,base=100000) {
  n <- length(p)
  R <- d/p
  qx <- delta*R/(1+0.5*delta*R)
  qx[n] <- 1
  lx <- base*c(1,cumprod(1-qx))
  d <- -diff(lx)
  lx <- lx[-(n+1)]
  return(lx)
}

delta <- c(1,4,rep(5,17))
xx <- seq(0,85,length=19)

attach(us.mortality)

lt60 <- life.table(pop[year==1960],deaths[year==1960],delta)
lt40 <- life.table(pop[year==1940],deaths[year==1940],delta)

par(mfrow=c(2,2))
plot(c(xx,90),c(lt60/lt60[1],0),type="l",ylim=c(0,1),
     xlab="age in years",ylab="survival probabilities")
abline(h=0); abline(v=0)
title("Life table -- US 1960")
plot(c(xx,90),c(lt40/lt40[1],0),type="l",ylim=c(0,1),
     xlab="age in years",ylab="survival probabilities")
abline(h=0); abline(v=0)
title("Life table -- US 1940")
newx <- c(xx,90)
newy60 <- c(lt60/lt60[1],0)
newy40 <- c(lt40/lt40[1],0)
plot(newx,newy60,type="l",ylim=c(0,1),
     xlab="age in years",ylab="survival probabilities")
lines(newx,newy40)
abline(h=0); abline(v=0)
polygon(c(newx,rev(newx)),c(newy60,rev(newy40)),density=15)
text(66,0.4,"difference in life expectancy",pos=2,cex=.8)
arrows(66,0.4,70.833,0.489,length=.05)
title("Life table -- US 1940/1960")

detach(us.mortality)
###

                                        # Fig. 8.4
t <- seq(0,60,0.1)
lam <- 0.03
s <- exp(-lam*t)
plot(t,s,type="l",ylim=c(0,1),xlab="time",ylab="",
     main="Survival data -- three measures")
abline(h=lam,lty=2)
lines(t,-log(s),lty=3)
legend(40,0.8,c("survival curve","hazard curve","cumulative hazard"),lty=1:3)

                                        # Fig. 8.5
###
### not in the book
st <- c(8,18,60,52,48,4,13,40,20,28)
censored <- c(1,2,10)
t <- c(92,82,3,20,30,40,60,20,17,72)
plot(seq(0,100),type="n",ylim=c(0,length(t)+1),xlim=c(0,110),
     xlab="time (months)",ylab="",axes=F)
axis(1,seq(0,100,by=20))
box()
abline(v=c(0,100))
segments(t,1:length(t),t+st,1:length(t))
points(t,1:length(t),pch=4)
points(t[-censored]+st[-censored],(1:length(t))[-censored],pch=16)
points(t[censored]+st[censored],(1:length(t))[censored],pch=1)
segments(t[censored]+st[censored],censored,t[censored]+st+10,censored,lty=3)
text((2*t+st)/2,1:length(t)+.2,st)
legend(5,2,c("start","complete","censored"),pch=c(4,16,1))
title("n = 10 observation -- three censored")
###

t0 <- c(4,8,13,18,20,28,40,48,52,60)
c0 <- c(1,0,1,0,1,0,1,1,1,1)
###
### - R --
library(survival)
summary(survfit(Surv(t0,c0)))
###
surv.fit(t0,c0)

                                        # Fig. 8.6
###
### - R --
plot(survfit(Surv(t0,c0)),conf.int=T,xlab="time",ylab="survival probability",
     main="Kaplan-Meier estimated survival curve")
###
plot(surv.fit(t0,c0),xlab="time",ylab="survival probability",
     conf.int=T,main="Kaplan-Meier estimated survival curve")
abline(h=0)

n.event <- surv.fit(t0,c0)$n.event[c0==1]
n.risk <- surv.fit(t0,c0)$n.risk[c0==1]
time <- surv.fit(t0,c0)$time[c0==1]
d.time <- diff(time)
k <- length(n.risk)
n.event <- n.event[-k]
n.risk <- n.risk[-k]
time <- time[-k]
h.rate <- n.event/(d.time*n.risk)
round(cbind(time,d.time,n.risk,n.event,h.rate),4)

surv <- surv.fit(t0,c0)$>surv
time <- surv.fit(t0,c0)$time
n.risk <- surv.fit(t0,c0)$n.risk
P <- c(1,surv[c0==1])
t <- c(0,time[c0==1],0)
tbar <- sum(P*diff(t))
n <- n.risk[c0==1]
fact <- 1/(n*(n-1))
fact <- c(ifelse(fact<Inf,fact,0),0)
a <- tbar-cumsum(P*diff(t))
se <- sqrt(sum(fact*a^2))
###
### not in the book
### correct some typographical errors (missing '=' and ')')
cat("mean survival time =",round(tbar,3),
    "standard error =",round(se,3),"\n")

d <- sum(c0)
tbar <- sum(t0)/d
rate <- 1/tbar
round(rate,3)
se <- sqrt(tbar^2/d)
cat("mean survival time =",round(tbar,3),
    "standard error =",round(se,3),"\n")

md <- funciton(t0,c0) {min(t0[surv.fit(t0,c0)$surv<=0.5])}
cat("estimated median =",md(t0,c0),"\n")

x <- 1:10
y <- x^2
xout <- x+0.5
yout <- approx(x,y,xout)$y
cbind(x,y,xout,yout)

md.linear <- function(t0,c0) {
  time <- surv.fit(t0,c0)$time[c0==1]
  p <- surv.fit(t0,c0)$surv[c0==1]
  approx(p,time,xout=0.5)$y
}

md.linear(t0,c0)

time <- c(9,13,13,18,23,28,31,34,45,48,161,5,5,8,8,12,16,23,27,30,33,43,45)
c0 <- c(1,1,0,1,1,0,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,1,1)
g0 <- rep(c(1,0),c(11,12))
###
### - R --
survfit(Surv(time,c0)~g0)
###
surv.fit(time,c0,g0)

                                        # Fig. 8.7
###
### - R --
plot(survfit(Surv(time,c0)~g0),conf.int=F,xlab="time (weeks)",lty=c(1,3),
     ylab="survival probability",
     main="Kaplan-Meier estimated survival curve -- two groups")
legend(60,0.8,c("maintained","nonmaintained"),lty=c(1,3))
###

###
### - R --
survdiff(Surv(time,c0)~g0)
###
surv.diff(time,c0,g0)

###
### - R --
coxph(Surv(time,c0)~g0)
###
coxreg(time,c0,g0)

###
### - R --
like <- coxph(Surv(time,c0)~g0)$loglik
###
like <- coxreg(time,c0,g0)$loglik
like
x2 <- -2*(like[1]-like[2])
pvalue <- 1-pchisq(x2,1)
cbind(x2,pvalue)


###
### not in the book
lung.c <- read.table("lungcancer.txt",header=T)
time <- lung.c$time
c0 <- lung.c$cens+1
c0 <- ifelse(c0==2,0,1)
g0 <- factor(lung.c$vital.capac,labels=c(0,1))
age <- lung.c$age
###

                                        # Fig. 8.8
###
### - R --
### replace surv.fit with R survif(Surv()) construction, and remove
### mark.time option
cplot <- function(id1,id2,id0) {
  ttmp <- time[age0==id1 & g0==id2]
  ctmp <- c0[age0==id1 & g0==id2]
  stmp <- g0[age0==id1 & g0==id2]
  par(new=T)
  plot(survfit(Surv(ttmp,ctmp)~stmp),conf.int=F,xlab="",ylab="",yaxt="n",
       xaxt="n",lty=id0)
}
plot(time,time,ylim=c(0,1),type="n",xlab="survival time (days)",
     ylab="survival probability",main="Survival and vital capacity")
age0 <- ifelse(age<66,1,2)
###
### not in the book
### correct second parameter which is not consistent with coding of factor
### (0/1 instead of 1/2)
cplot(1,0,1)
cplot(1,1,2)
cplot(2,0,3)
cplot(2,1,4)
legend(250,.9,c("<= 65 & High","<= 65 & Low","> 65 & High","> 65 & Low"),
       lty=1:4)

###
### - R --
coxph(Surv(time,cens)~age+vital.capac,data=lung.c)
###
coxreg(time,c0,cbind(age,g0))

###
### - R --
coxph(Surv(time,cens)~vital.capac,data=lung.c)
###

coxreg(time,c0,g0)

like1 <- coxreg(time,c0,cbind(age,g0))$loglik
like0 <- coxreg(time,c0,g0)$loglik
x2 <- -2*(like0[2]-like1[2])
pvalue <- 1-pchisq(x2,1)
cbind(x2,pvalue)

like0 <- coxreg(time,c0,age)$loglik
x2 <- -2*(like0[2]-like1[2])
pvalue <- 1-pchisq(x2,1)
cbind(x2,pvalue)

temp <- scan("aids.data")
time <- temp[seq(1,length(temp),4)]
c0 <- temp[seq(2,length(temp),4)]

                                        # Fig. 8.9
###
### - R --
plot(survfit(Surv(time,c0)),xlab="time (months)",ylab="survival probability",
     conf.int=T,main="Kaplan-Meier survival curve -- time to aids")
###
plot(surv.fit(time,c0),xlab="time (months)",ylab="survival probability",
     conf.int=T,main="Kaplan-Meier survival curve -- time to aids")

temp <- scan("aids.data")
time <- temp[seq(1,length(temp),4)]
c0 <- temp[seq(2,length(temp),4)]
beta <- temp[seq(3,length(temp),4)]
cd4 <- temp[seq(4,length(temp),4)]
cd4bybeta <- beta*cd4
x <- cbind(beta,cd4,cd4bybeta)

###
### - R --
coxph(Surv(time,c0)~x)
###
coxreg(time,c0,x)

###
### - R --
coxph(Surv(time,c0)~x[,-3])
###
coxreg(time,c0,x[,-3])

like <- coxreg(time,c0,x)$loglik
like0 <- coxreg(time,c0,x[,-3])$loglik
x2 <- -2*(like0[2]-like[2])
pvalue <- 1-pchisq(x2,1)
cbind(x2,pvalue)

like1 <- coxreg(time,c0,x[,-3])$loglik
like0 <- coxreg(time,c0,cd4)$loglik
x2 <- -2*(like0[2]-like1[2])
pvalue <- 1-pchisq(x2,1)
cbind(x2,pvalue)

like0 <- coxreg(time,c0,beta)$loglik
x2 <- -2*(like0[2]-like1[2])
pvalue <- 1-pchisq(x2,1)
cbind(x2,pvalue)

                                        # Fig. 8.10
otemp <- coxreg(time,c0,x[,-3],resid="schoen")
###
### - R --
otemp <- coxph(Surv(time,c0)~x[,-3])
rcox <- residuals(otemp,type="schoenfeld")
tcox <- as.numeric(rownames(rcox))
###
rcox <- otemp$resid
tcox <- otemp$time
par(mfrow=c(2,1))
labels <- c("beta level","cd4 count")
for (i in 1:2) {
  plot(tcox,rcox[,i],xlab="time",ylab="Schoenfeld residuals",pch=i,
       main=paste("Residual values -- ",labels[i]))
  abline(h=0)
  lines(tcox,smooth(rcox[,i]),lty=2)
  abline(lsfit(tcox,rcox[,i]))
}

round(cor(rcox,tcox),3)

###
### not in the book
otemp1 <- coxph(Surv(time,c0)~x[,-3],subset=time>=30 & time <= 85)
rcox1 <- residuals(otemp1,type="schoenfeld")
tcox1 <- as.numeric(rownames(rcox1))
###
round(cor(rcox1,tcox1),3)

age0 <- rep(c(0,1),c(52,37))
###
### - R --
coxph(Surv(time,c0)~x[,-3]+strata(age0))
###
coxreg(time,c0,x[,-3],strata=age0)

                                        # Fig. 8.11
###
### not in the book
plot(survfit(Surv(time,c0)~age0),xlab="survival time",
     ylab="probability of surviving",lty=c(1,3),conf.int=F)
legend(20,0.4,c("age less than 30","age greater or equal 30"),lty=c(1,3))
title("Proportional hazards\n stratified by age")
###
